import json
import os
import tarfile
from io import BytesIO
from zipfile import ZipFile

import httpx

from satoricli.api import client

from ..utils import console, error_console, wait


def run_script(
    path: str, team: str, visibility: str | None, show_stdout: bool, settings: dict
):
    with BytesIO() as bundle:
        with ZipFile(bundle, "x") as zf:
            zf.writestr(".satori.yml", f"cmd: [ sh '{os.path.basename(path)}' ]")

        bundle.seek(0)

        data = client.post(
            "/runs",
            data={
                "path": path,
                "with_files": True,
                "save_report": True,
                "save_output": True,
                "team": team,
                "visibility": visibility.capitalize() if visibility else None,
                "settings": json.dumps(settings),
            },
            files={"bundle": bundle},
        ).json()

    report_id = data["report_ids"][0]
    arc = data["upload_data"]

    error_console.print("Report ID:", report_id)
    error_console.print(f"Report: https://satori.ci/report/{report_id}")

    with BytesIO() as packet:
        with tarfile.open(fileobj=packet, mode="w:gz") as tf:
            info = tf.gettarinfo(path, os.path.basename(path))
            info.mode = 0o755

            with open(path, "rb") as script:
                tf.addfile(info, script)

        packet.seek(0)

        res = httpx.post(arc["url"], data=arc["fields"], files={"file": packet})
        res.raise_for_status()

    if show_stdout:
        wait(report_id)

        for output in client.get(f"/outputs/{report_id}").json():
            stdout = output.get("output", {}).get("stdout")

            if stdout is not None:
                console.out(stdout, highlight=False)



import re
from argparse import ArgumentParser
from typing import Any, Literal, get_args

from rich.markdown import Markdown
from rich.prompt import Prompt
from rich.syntax import Syntax
from rich.table import Table

from satoricli.api import client
from satoricli.cli.utils import (
    add_table_row,
    autoformat,
    autosyntax,
    console,
    date_formatter,
    download_files,
    error_console,
    execution_time,
    get_command_params,
    print_output,
    wait,
)

from .base import BaseCommand

VISIBILITY_VALUES = ("public", "private", "unlisted")
ACTIONS = Literal[
    "show",
    "output",
    "stop",
    "files",
    "delete",
    "status",
    "set-team",
    "visibility",
    "issue",
]


class ReportCommand(BaseCommand):
    name = "report"

    def register_args(self, parser: ArgumentParser):
        parser.add_argument("id", metavar="ID")
        parser.add_argument(
            "action",
            metavar="ACTION",
            choices=get_args(ACTIONS),
            nargs="?",
            default="show",
            help="action to perform",
        )
        parser.add_argument("action2", nargs="?", default=None)
        parser.add_argument(
            "--test",
            dest="filter_tests",
            help="Print specified test output",
            action="append",
            default=[],
        )
        parser.add_argument(
            "--format",
            dest="text_format",
            help="Format text output (Plain or Markdown text)",
            default="plain",
            choices=("plain", "md"),
        )
        parser.add_argument(
            "--unredacted", action="store_true", help="Unredacted output"
        )
        parser.add_argument("--query", help="Search query for issue template")
        parser.add_argument("--title", help="Title for the issue")

    def __call__(
        self,
        id: str,
        action: ACTIONS,
        action2: str,
        filter_tests: list[str],
        text_format: Literal["plain", "md"],
        unredacted: bool,
        query: str,
        title: str,
        **kwargs,
    ) -> int:
        if action == "show":
            if kwargs["team"]:
                # Set team for the report
                client.patch(
                    f"/reports/{id}/team",
                    params={"team_name": kwargs["team"]},
                ).json()
            res = client.get(f"/reports/{id}", params={"unredacted": unredacted}).json()
            if kwargs["json"]:
                autoformat(res, jsonfmt=kwargs["json"])
            else:
                ReportCommand.print_report_single(res)
        elif action == "output":
            status = client.get(f"/reports/{id}/status").text

            if status == "Running":
                wait(id, True)
            else:
                print_output(id, kwargs["json"], filter_tests, text_format, unredacted)
        elif action == "files":
            download_files(id)
        elif action == "stop":
            res = client.patch(f"/reports/{id}/stop").json()
            autoformat(res, jsonfmt=kwargs["json"])
        elif action == "delete":
            client.delete(f"/reports/{id}")
            console.print("Report deleted")
        elif action == "visibility":
            if not action2 or action2 not in VISIBILITY_VALUES:
                error_console.print(
                    f"Allowed values for visibility: {VISIBILITY_VALUES}",
                )
                return 1
            res = client.patch(
                f"/reports/{id}",
                json={"visibility": action2.capitalize()},
            ).json()
            autoformat(res)
        elif action == "status":
            res = client.get(f"/reports/{id}/status").text
            autoformat(res)
        elif action == "set-team":
            res = client.patch(
                f"/reports/{id}/team",
                params={"team_name": action2},
            ).json()
            autoformat(res)
        elif action == "issue":
            if not action2:
                error_console.print("Please specify an issue template ID")
                return 1
            res = client.get(
                f"/reports/{id}/issue/{action2}",
                params={"q": query},
            ).json()
            if kwargs["json"]:
                autoformat(res, jsonfmt=kwargs["json"])
            else:
                display_title = title or res["title"]
                md = Markdown(f"# {display_title}\n\n{res['body']}")
                console.print(md)
            p = Prompt.ask(
                f"Do you want to publish this issue on [b]{res['repository']}[/]?",
                choices=["y", "n"],
            )
            if p == "y":
                _ = client.post(
                    f"/reports/{id}/issue/{action2}",
                    params={"repository": res["repository"], "title": title},
                )
                console.print("Issue published")
        return 0

    @staticmethod
    def print_report_list(report_list: list[dict[str, Any]]) -> None:
        """Print reports list as a table.

        Parameters
        ----------
        report_list : list
            Reports list

        """
        for report in report_list:
            table = Table(
                show_header=False, show_lines=True, highlight=True, expand=False
            )
            add_table_row(
                [
                    ["ID", report["id"]],
                    ["Team", report["team"], "cyan1"],
                    ["Playbook name", report["playbook_name"], "cyan1"],
                    ["Report URL", report.get("report_url"), "blue"],
                ],
                table,
            )
            add_table_row(
                [
                    ["Execution type", report["execution"], "bright_magenta"],
                    ["Visibility", report["visibility"]],
                    ["Runtime", execution_time(report.get("execution_time"))],
                    ["User", report["user"]],
                    ["Date", date_formatter(report["date"])],
                    ["Status", report["status"]],
                    ["Result", report["result"]],
                ],
                table,
            )

            if report.get("repo"):
                add_table_row(
                    [
                        ["Repo", report["repo"]],
                        ["Branch", report["branches"]],
                        ["Hash", report["hash"]],
                        ["Commit author", report["commit_author"]],
                    ],
                    table,
                )

            # Playbooks Line
            playbook_line = []
            if report["playbook_path"]:
                playbook_line.append(["Path", report["playbook_path"]])
            if report["secrets_count"]:
                playbook_line.append(["Parameters", report["secrets_count"]])
            if playbook_line:
                add_table_row(playbook_line, table)

            if report["testcases"]:
                tests = ""
                for test in report["testcases"]:
                    style = "[pass]" if re.search(r"pass$", test) else "[fail]"
                    tests += f"\n  ○ {style}{test}"
                table.add_row(f"[b]Testcases:[/]{tests}")
            if report["errors"]:
                table.add_row(
                    "[warning]Warnings and Errors:[/]\n ○ "
                    + str(report["errors"]).replace("\n", "\n ○ ")
                )
            console.print(table, "\n")

    @staticmethod
    def print_report_single(report: dict):
        """Print a single report as a table.

        Parameters
        ----------
        report : dict
            Report data
        """
        table = Table(show_header=False, show_lines=True, highlight=True, expand=False)
        # Create a row with the basic info
        add_table_row(
            [
                ["ID", report["id"]],
                ["Team", report["team"], "cyan1"],
                ["Playbook name", report["name"], "cyan1"],
                ["Report url", report.get("report_url"), "blue"],
            ],
            table,
        )
        # Add another row with the report data
        add_table_row(
            [
                ["Execution type", report["execution"], "bright_magenta"],
                ["Visibility", report["visibility"]],
                ["Source", report["source"]],
                ["Runtime", execution_time(report.get("execution_time"))],
                ["Monitor", report["monitor_id"]],
                ["Date", date_formatter(report["created"])],
                ["Status", report["status"]],
                ["Result", report["result"]],
            ],
            table,
        )
        if report["run_params"]:
            add_table_row(
                [["Command", "[green1]satori " + report["run_params"]]],
                table,
            )

        if report.get("repo"):
            # Add the repo data in another row if exist
            add_table_row(
                [
                    ["Repo", report["repo"]],
                    ["Branch", report["branches"]],
                    ["Hash", report["hash"]],
                    ["Parent", report.get("parent_hash")],
                ],
                table,
            )

        # Add the playbook data in a new row
        playbook_line = []
        if report["playbook_path"]:
            playbook_line.append(["Path", report["playbook_path"]])
        if report["secrets_count"]:
            playbook_line.append(["Secrets Count", report["secrets_count"]])
        params = get_command_params(report["run_params"])
        if params:
            playbook_line.append(["Parameters", params])
        if len(playbook_line) > 0:
            add_table_row(playbook_line, table)

        # Highlight the playbook content
        playbook_content = autosyntax(report["playbook"], echo=False)
        if isinstance(playbook_content, Syntax):
            table.add_row(playbook_content)
        if report["testcases"]:
            tests = ""
            for test in report["testcases"]:
                style = "[pass]" if re.search(r"pass$", test) else "[fail]"
                tests += f"\n  ○ {style}{test}"
            table.add_row(f"[b]Testcases:[/]{tests}")
        if report["report"]:
            ReportCommand.print_report_summary(report["report"], table)
        if report["delta"]:
            table.add_row(f"[b]Report:[/]\n{autoformat(report['delta'], echo=False)}")
        if report["user_warnings"]:
            table.add_row(
                "[warning]Warnings and Errors:[/]\n ○ "
                + str(report["user_warnings"]).replace("\n", "\n ○ ")
            )
        console.print(table)  # Print the table content

    @staticmethod
    def print_report_summary(report_data: list, table: Table):
        """Print the json content of a report.

        Parameters
        ----------
        report_data : list
            Report json
        table : Table
            Table to add the content
        """
        for report in report_data:
            if not report["asserts"]:
                continue
            assert_props = [
                ["Test", report["test"]],
            ]
            if report.get("severity"):
                assert_props.append(["Severity", report["severity"]])
            test_status = (
                report["test_status"]
                if not report["total_fails"]
                else f"Fail({report['total_fails']})"
            )
            assert_props.extend([["Test status", test_status]])
            row = add_table_row(assert_props, table, echo=False) or ""
            for ast in report["asserts"]:
                assert_status = (
                    ast["status"] if not ast["count"] else f"Fail({ast['count']})"
                )
                row += "\n" + (
                    add_table_row(
                        [
                            ["Assert", ast["assert"]],
                            ["Assert Value", str(ast["expected"]).strip()],
                            ["Status", assert_status],
                        ],
                        table,
                        echo=False,
                    )
                    or ""
                )
            table.add_row(row)

    @staticmethod
    def print_report_asrt(report_id: str, json_out: bool):
        """Fetch the report and print the report summary.

        Parameters
        ----------
        report_id : str
            ID of the report
        json_out : bool
            Print as json?
        """
        # Fetch the report data
        report_data = client.get(f"/reports/{report_id}").json()
        json_data = report_data.get("report")
        if json_data:
            if json_out:  # Print as json if --json is defined
                console.print_json(data=json_data)
            else:
                table = Table(
                    show_header=False, show_lines=True, highlight=True, expand=False
                )
                add_table_row(
                    [["Result", report_data["result"]]],
                    table,
                )

                # Add the parameters if they exist
                params = get_command_params(report_data["run_params"])
                if params:
                    add_table_row([["Parameters", params]], table)

                ReportCommand.print_report_summary(report_data["report"], table)
                console.print(table)
        elif report_data["status"] == "Timeout":
            console.print("[error]Report timed out")
        else:
            console.print("[error]Report not found")



import shutil
import subprocess
from argparse import ArgumentParser

from ..utils import error_console
from .base import BaseCommand

PROMPT = """\
You are Loop, the Satori CI assistant. Satori is an automated testing platform \
that uses YAML playbooks to test any software, system, or network inside containers. \
Tests run on demand, on CI (GitHub push), or on a schedule (monitors).

# Setup
On first interaction, ensure the satorici GitHub repos (playbooks, playbook-validator, \
satori-cli, satori-docs) are cloned (--depth 1) or pulled under ~/.satori/. \
Use these repos as your knowledge base for examples and validation rules.

# Playbook Syntax
Playbooks are YAML files with this structure:

```yaml
settings:
  name: "Playbook Name"
  description: "What it tests"
  image: debian  # Docker base image
  # Optional: timeout, cpu, memory, cron, rate, os, storage

import:  # optional
  - satori://path/to/playbook.yml

test_block_name:
  install:
    - apt-get install -qy tool
  run:
    - tool --target ${{PARAM}}
  assertReturnCode: 0
  assertStdoutNotContains: "ERROR"
```

## Variables
- `${{VAR}}` — user provides via `satori run -d VAR=value`
- `${{step.stdout}}` — reference output from a previous step

## Assertions (apply to the command group they're in)
- assertReturnCode / assertReturnCodeNot (integer)
- assertStdout / assertStderr (true|false — was there output?)
- assertStdoutEqual / assertStdoutNotEqual (exact match)
- assertStdoutContains / assertStdoutNotContains (substring, accepts array)
- assertStdoutRegex / assertStdoutNotRegex (regex pattern)
- assertStderrEqual / assertStderrNotEqual / assertStderrContains / assertStderrNotContains
- assertStderrRegex / assertStderrNotRegex
- assertDifferent (true|false), assertKilled (true|false)
- setSeverity: 1-5 (1=critical, 5=info)

## Inputs (parameterization & fuzzing)
```yaml
input:
  - - "value1"
    - "value2"
run:
  - cmd ${{input}}  # runs once per value
```
Supports file-based inputs (`file: dict.txt, split: "\\n"`) and mutations (`mutate: radamsa`).

# Key CLI Commands
- `satori run playbook.yml --report --output` — execute remotely (add `-d KEY=VAL` for params) and show the report and the output
- `satori run playbook.yml --local` — execute locally
- `satori run satori://code/semgrep.yml` — run a public playbook
- `satori shell --image debian` — interactive dev shell
- `satori monitor` — manage scheduled executions
- `satori scan` — scan repos across commits
- `satori report <id>` — view execution results

# Guidelines
- If an assertion is required, include it in the playbook.
- Write playbooks in the current working directory unless told otherwise.
- Prefer simple, minimal playbooks.
- When the user describes what to test, produce a ready-to-run .yml file and provide the command to execute it.
- Validate generated playbooks against the schemas in ~/.satori/playbook-validator.
"""


class AiCommand(BaseCommand):
    name = "ai"

    def register_args(self, parser: ArgumentParser):
        parser.add_argument(
            "prompt",
            metavar="PROMPT",
            nargs="?",
            default=None,
            help="Optional initial prompt (omit for interactive mode)",
        )

    def __call__(self, prompt: str | None = None, **kwargs):
        if not shutil.which("claude"):
            error_console.print("ERROR: claude CLI is not installed.")
            error_console.print("Install it: npm install -g @anthropic-ai/claude-code")
            return 1

        cmd = ["claude", "--dangerously-skip-permissions", "--system-prompt", PROMPT]
        if prompt:
            cmd += ["-p", prompt]

        result = subprocess.run(cmd, check=False)
        return result.returncode



import json
import os
import shutil
import sys
import tempfile
import time
import uuid
import warnings
from argparse import ArgumentParser
from pathlib import Path
from typing import Any, Literal, Optional, Union, get_args

import httpx
import yaml
from rich.progress import open as progress_open
from satorici.validator import validate_settings

from satoricli.api import client
from satoricli.bundler import make_bundle
from satoricli.cli.commands.run_script import run_script
from satoricli.cli.commands.scan import ScanCommand
from satoricli.cli.utils import log
from satoricli.validations import get_parameters, validate_parameters

from ..utils import (
    console,
    detect_boolean,
    download_files,
    error_console,
    get_report_output_cmd_index,
    load_cli_params,
    missing_ymls,
    print_output,
    print_summary,
    tuple_to_dict,
    validate_config,
    wait,
)
from .base import BaseCommand
from .report import ReportCommand

VISIBILITY_VALUES = Literal["public", "private", "unlisted"]


def include_files(include: list[str]) -> Optional[str]:
    if not include:
        return
    temp_dir = Path(tempfile.gettempdir(), str(uuid.uuid4()))
    os.makedirs(temp_dir, exist_ok=True)
    for file in include:
        shutil.copy(file, temp_dir)
    shutil.make_archive(str(temp_dir), "gztar", temp_dir)
    return f"{temp_dir}.tar.gz"


def make_packet(path: str):
    temp_file = Path(tempfile.gettempdir(), str(uuid.uuid4()))
    shutil.make_archive(str(temp_file), "gztar", path)
    return f"{temp_file}.tar.gz"


def new_run(
    *,
    path: str,
    team: str,
    modes: Optional[dict] = None,
    bundle: Optional[Any] = None,
    packet: Optional[Path] = None,
    secrets: Optional[dict] = None,
    settings: Optional[dict] = None,
    save_report: Union[str, bool, None] = None,
    save_output: Union[str, bool, None] = None,
    params: Union[str, None] = None,
    visibility: Optional[VISIBILITY_VALUES] = None,
    clone: Optional[str] = None,
    redacted: list[str] = [],
) -> list[str]:
    data = client.post(
        "/runs",
        data={
            "path": path,
            "secrets": json.dumps(secrets) if secrets else None,
            "settings": json.dumps(settings) if settings else None,
            "with_files": bool(packet),
            "modes": json.dumps(modes) if modes else None,
            "save_report": save_report,
            "save_output": save_output,
            "team": team,
            "run_params": " ".join(sys.argv[1:]),
            "visibility": visibility.capitalize() if visibility else None,
            "clone": clone,
            "redacted": redacted,
        },
        files={"bundle": bundle} if bundle else {"": ""},
    ).json()

    if arc := data["upload_data"]:
        if arc.get("type") == "scan":
            reports_list = []
            while not reports_list:
                res = client.get(f"/scan/{data['report_ids'][0]}/reports").json()
                reports_list = res.get("rows")
                time.sleep(1)
            return [x["id"] for x in reports_list]
        try:
            with progress_open(
                packet, "rb", description="Uploading...", console=error_console
            ) as f:
                res = httpx.post(arc["url"], data=arc["fields"], files={"file": f})
            res.raise_for_status()
        finally:
            os.remove(packet)

    return data["report_ids"]


def new_monitor(
    bundle,
    settings: dict,
    team: str,
    *,
    packet: Optional[Path] = None,
    secrets: Optional[dict] = None,
    visibility: Optional[VISIBILITY_VALUES] = None,
) -> str:
    data = client.post(
        "/monitors",
        data={
            "secrets": json.dumps(secrets) if secrets else None,
            "settings": json.dumps(settings),
            "with_files": bool(packet),
            "team": team,
            "visibility": visibility.capitalize() if visibility else "Private",
        },
        files={"bundle": bundle} if bundle else {"": ""},
    ).json()

    if arc := data["upload_data"]:
        try:
            with progress_open(
                packet, "rb", description="Uploading...", console=error_console
            ) as f:
                res = httpx.post(arc["url"], data=arc["fields"], files={"file": f})
            res.raise_for_status()
        finally:
            os.remove(packet)

    return data["monitor_id"]


def has_files(playbook_path: Path):
    try:
        return yaml.safe_load(playbook_path.read_text())["settings"]["files"]
    except Exception:
        return False


def get_cli_settings(args: dict):
    keys = (
        "cpu",
        "memory",
        "cron",
        "rate",
        "count",
        "name",
        "timeout",
        "os",
        "image",
        "storage",
    )

    return {key: value for key, value in args.items() if key in keys and value}


def warn_settings(settings: dict):
    with warnings.catch_warnings(record=True) as ws:
        validate_settings(settings)

        for w in ws:
            log.warning(w.message)


def get_parameters_from_env(playbook_path: str):
    with open(playbook_path) as f:
        playbook: dict = yaml.safe_load(f)
    return {k: [v] for k, v in os.environ.items() if k in get_parameters(playbook)}


class RunCommand(BaseCommand):
    name = "run"

    def register_args(self, parser: ArgumentParser):
        parser.add_argument("path", metavar="PATH", default=".", nargs="?")
        parser.add_argument(
            "-d", "--data", type=load_cli_params, action="append", default=[]
        )
        parser.add_argument(
            "-p",
            "--playbook",
            help="if PATH is a directory this playbook will be used",
        )
        parser.add_argument("--save-report", type=str, default=None)
        parser.add_argument("--save-output", type=str, default=None)
        parser.add_argument("--clone", type=str, default=None)
        parser.add_argument(
            "-df", "--data-file", type=load_cli_params, action="append", default=[]
        )
        parser.add_argument(
            "-i", "--include", dest="include_list", action="append", default=[]
        )
        parser.add_argument("--repo", type=str, default=None, help="Repository to run")

        settings = parser.add_argument_group("run settings")
        monitor = settings.add_mutually_exclusive_group()
        monitor.add_argument("--rate")
        monitor.add_argument("--cron")
        settings.add_argument("--name")
        settings.add_argument("--count", type=int)
        settings.add_argument("--cpu", type=int)
        settings.add_argument("--memory", type=int)
        settings.add_argument("--storage", type=int)
        settings.add_argument("--timeout", type=int)
        settings.add_argument("--os", choices=("windows", "linux"))
        settings.add_argument("--image")

        sync = parser.add_argument_group("sync run")
        sync.add_argument("-s", "--sync", action="store_true")
        sync.add_argument("-o", "--output", action="store_true")
        sync.add_argument("-r", "--report", action="store_true")
        sync.add_argument("-f", "--files", action="store_true")
        sync.add_argument("--stdout", action="store_true")
        parser.add_argument(
            "--visibility", choices=get_args(VISIBILITY_VALUES), default=None
        )
        parser.add_argument(
            "--test",
            dest="filter_tests",
            help="Print specified test output",
            action="append",
            default=[],
        )
        parser.add_argument(
            "--format",
            dest="text_format",
            help="Format text output (Plain or Markdown text)",
            default="plain",
            choices=("plain", "md"),
        )
        parser.add_argument(
            "--redacted",
            dest="redacted",
            help="Redacted parameters",
            action="append",
            default=[],
        )

    def __call__(
        self,
        path: str,
        sync: bool,
        data: list[tuple[str, str]],
        data_file: list[tuple[str, str]],
        include_list: list,
        repo: Optional[str],
        save_report: Union[str, bool, None],
        save_output: Union[str, bool, None],
        playbook: Optional[str],
        output: bool,
        report: bool,
        files: bool,
        stdout: bool,
        team: str,
        filter_tests: list,
        text_format: Literal["plain", "md"],
        visibility: Optional[VISIBILITY_VALUES] = None,
        clone: Optional[str] = None,
        redacted: list[str] = [],
        **kwargs,
    ):
        cli_settings = get_cli_settings(kwargs)

        if path.endswith(".sh"):
            run_script(path, team, visibility, stdout, cli_settings)
            return

        for file in data_file:
            read_path = Path(file[1])
            read_name = read_path.name
            data.append((file[0], f"read({read_name})"))
            include_list.append(file[1])
        if save_report:
            temp_report = detect_boolean(save_report)
            save_report = save_report if temp_report is None else temp_report
        if save_output:
            save_output = detect_boolean(save_output)
        modes = {"sync": sync, "output": output, "report": report}

        parsed_data = tuple_to_dict(data) if data else None
        if parsed_data and not validate_parameters(parsed_data):
            raise ValueError("Malformed parameters")

        is_monitor = bool(cli_settings.get("rate") or cli_settings.get("cron"))

        ids = []
        monitor_id = None

        if repo:
            if not parsed_data:
                parsed_data = {}
            parsed_data["REPO"] = repo
            if path != "." and not playbook:
                playbook = path
                path = "."

        if path == "." and parsed_data and "REPO" in parsed_data:
            # Check if playbook is a file (not a uri) and read it
            if playbook and "://" not in playbook and os.path.isfile(playbook):
                playbook = Path(playbook).read_text()

            params = {
                "url": parsed_data["REPO"],
                "data": json.dumps(parsed_data),
                "playbook": playbook,
                "branch": "main",
                "team": team,
                "run_params": " ".join(sys.argv[1:]),
                "run_last": True,
                "visibility": visibility.capitalize() if visibility else "Private",
            }
            info = client.post("/scan", json=params).json()
            report = sync if sync else report
            return ScanCommand.scan_sync(
                info["id"],
                kwargs,
                output,
                report,
                filter_tests,
                text_format,
                total_commits=info["total"],
            )

        if playbook and "://" not in playbook and not os.path.isfile(playbook):
            error_console.print("ERROR: Invalid playbook arg.")
            return 1

        if "://" in path:
            with warnings.catch_warnings(record=True):
                validate_settings(cli_settings)
            ids = new_run(
                path=path,
                team=team,
                modes=modes,
                secrets=parsed_data,
                settings=cli_settings,
                save_report=save_report,
                save_output=save_output,
                visibility=visibility,
                clone=clone,
                packet=include_files(include_list),
                redacted=redacted,
            )

            is_monitor = False
        elif (file_path := Path(path)).is_file():
            # HOOK HERE
            provided_var_names = set(parsed_data.keys()) if parsed_data else set()
            env_vars = get_parameters_from_env(file_path)

            if not validate_config(file_path, set(env_vars) | provided_var_names):
                return 1

            bundle = make_bundle(file_path, file_path.parent)
            config = yaml.safe_load(file_path.read_bytes())

            settings: dict[str, Any] = config.get("settings", {})
            is_monitor = is_monitor or settings.get("cron") or settings.get("rate")
            settings.update(cli_settings)

            warn_settings(settings)

            secrets = env_vars | (parsed_data or {})

            if is_monitor:
                monitor_id = new_monitor(
                    bundle, settings, team, secrets=secrets, visibility=visibility
                )
            else:
                ids = new_run(
                    path=path,
                    team=team,
                    modes=modes,
                    bundle=bundle,
                    secrets=secrets,
                    settings=settings,
                    save_report=save_report,
                    save_output=save_output,
                    visibility=visibility,
                    clone=clone,
                    packet=include_files(include_list),
                    redacted=redacted,
                )
        elif (base := Path(path)).is_dir():
            settings = {}
            packet = make_packet(base)
            bundle = None
            secrets = None

            if playbook and "://" in playbook:
                playbook_path = playbook
            else:
                if playbook and os.path.isfile(playbook):
                    dir_playbook = Path(playbook)
                elif (base / ".satori.yml").is_file():
                    dir_playbook = base / ".satori.yml"
                else:
                    error_console.print("ERROR: No playbook found.")
                    return 1

                playbook_path = str(dir_playbook)

                config = yaml.safe_load(dir_playbook.read_bytes())

                settings: dict = config.get("settings", {})
                is_monitor = is_monitor or settings.get("cron") or settings.get("rate")
                settings.update(cli_settings)

                warn_settings(settings)

                provided_var_names = set(parsed_data.keys()) if parsed_data else set()
                env_vars = get_parameters_from_env(playbook_path)

                # HOOK HERE
                if not validate_config(
                    dir_playbook, set(env_vars) | provided_var_names
                ):
                    return 1

                if missing_ymls(config, path):
                    log.warning(
                        "There are some .satori.yml outside the root "
                        "folder that have not been imported."
                    )

                bundle = make_bundle(dir_playbook, dir_playbook.parent)

            if is_monitor:
                monitor_id = new_monitor(
                    bundle,
                    settings,
                    team,
                    packet=packet,
                    secrets=secrets or parsed_data,
                    visibility=visibility,
                )
            else:
                ids = new_run(
                    path=playbook_path,
                    team=team,
                    modes=modes,
                    bundle=bundle,
                    secrets=secrets or parsed_data,
                    packet=packet,
                    settings=settings,
                    save_report=save_report,
                    save_output=save_output,
                    visibility=visibility,
                    clone=clone,
                    redacted=redacted,
                )
        else:
            error_console.print("ERROR: Invalid PATH")
            return 1

        if not is_monitor and not kwargs["json"]:
            for report_id in ids:
                error_console.print("Report ID:", report_id)
                error_console.print(f"Report: https://satori.ci/report/{report_id}")
        elif not is_monitor and kwargs["json"]:
            console.print_json(data={"ids": ids})
        elif is_monitor and not kwargs["json"]:
            console.print("Monitor ID:", monitor_id)
            console.print(f"Monitor: https://satori.ci/monitor/{monitor_id}")
            return
        elif is_monitor and kwargs["json"]:
            console.print_json(data={"monitor_id": monitor_id})
            return

        if sync or report or output or files:
            wait(ids[0], not report, filter_tests, text_format)

        ret = print_summary(ids[0], kwargs["json"]) if sync else 0

        report_index, output_index = get_report_output_cmd_index()
        should_print_output = (output and bool(kwargs["json"])) or (report and output)
        print_args: tuple[str, bool] = (ids[0], kwargs["json"])
        if output_index < report_index:
            if should_print_output:
                print_output(*print_args)
            if report:
                ReportCommand.print_report_asrt(*print_args)
        else:
            if report:
                ReportCommand.print_report_asrt(*print_args)
            if should_print_output:
                print_output(*print_args)

        if files:
            download_files(ids[0])

        return ret



import shutil
import sys
from argparse import ArgumentParser
from typing import Optional

from satoricli.api import client
from satoricli.cli.utils import autoformat, autotable

from ..utils import BootstrapTable, console, get_offset
from .base import BaseCommand


class PlaybooksCommand(BaseCommand):
    name = "playbooks"

    @staticmethod
    def truncate_image_name(image_name: str, max_length: int = 29) -> str:
        """Truncate image name to specified length, adding ... if necessary"""
        if not image_name:
            return ""
        elif len(image_name) <= max_length:
            return image_name.ljust(max_length)
        return image_name[:max_length-3] + "..."

    @staticmethod
    def is_piped_output():
        """Check if output is being piped or redirected"""
        return not sys.stdout.isatty()

    @staticmethod
    def calculate_widths(sast_rows, dast_rows):
        """Calculate widths ensuring URI and Parameters are never truncated"""
        is_piped = PlaybooksCommand.is_piped_output()
        
        all_rows = sast_rows + dast_rows
        uri_width = max(len(row["uri"]) for row in all_rows) if all_rows else 30
        
        max_params_length = 0
        if dast_rows:
            raw_params_length = max(len("\n".join(row.get("parameters", []))) for row in dast_rows)
            max_params_length = min(raw_params_length, 30)
        
        if is_piped:
            estimated_width = 160
            
            MIN_IMAGE_WIDTH = 15
            MIN_NAME_WIDTH = 15
            
            BORDER_PADDING = 3
            SAST_PADDING = BORDER_PADDING * 3
            DAST_PADDING = BORDER_PADDING * 4
            
            dast_available = estimated_width - uri_width - max_params_length - DAST_PADDING
            
            if dast_available < (MIN_IMAGE_WIDTH + MIN_NAME_WIDTH):
                image_width = MIN_IMAGE_WIDTH
                name_width = MIN_NAME_WIDTH
            else:
                image_width = int(min(MIN_IMAGE_WIDTH * 2, dast_available * 0.2))
                name_width = int(dast_available - image_width)
            
            sast_name_width = name_width + max_params_length
            
            return {
                'sast': (uri_width, sast_name_width, image_width),
                'dast': (uri_width, name_width, max_params_length, image_width)
            }
        
        terminal_width = shutil.get_terminal_size().columns
        
        MIN_IMAGE_WIDTH = 10
        MIN_NAME_WIDTH = 10
        
        BORDER_PADDING = 3
        SAST_PADDING = BORDER_PADDING * 3
        DAST_PADDING = BORDER_PADDING * 4
        
        available_width = terminal_width - uri_width - max_params_length - max(SAST_PADDING, DAST_PADDING)
        
        if available_width < 0:
            image_width = MIN_IMAGE_WIDTH
            name_width = max(MIN_NAME_WIDTH, terminal_width - uri_width - max_params_length - DAST_PADDING - image_width)
        else:
            image_width_sast = max(MIN_IMAGE_WIDTH, min(available_width // 3, 25)) +1
            image_width_dast = max(MIN_IMAGE_WIDTH, min(available_width // 3, 25)) +1
            
            name_width = max(MIN_NAME_WIDTH, available_width - image_width_dast) - 2
        
        sast_name_width = name_width + max_params_length
        
        return {
            'sast': (uri_width, sast_name_width, image_width_sast),
            'dast': (uri_width, name_width, max_params_length, image_width_dast)
        }

    def register_args(self, parser: ArgumentParser):
        parser.add_argument(
            "-p",
            "--page",
            dest="page",
            type=int,
            default=1,
            help="Playbooks page number",
        )
        parser.add_argument(
            "-l",
            "--limit",
            dest="limit",
            type=int,
            default=20,
            help="Page limit number",
        )
        parser.add_argument(
            "--public", action="store_true", help="Fetch public satori playbooks"
        )
        parser.add_argument("--monitor", help="A bool value")

    def __call__(
        self, page: int, limit: int, public: bool, monitor: Optional[str], **kwargs
    ):
        offset = get_offset(page, limit)
        params: dict = {"offset": offset, "limit": limit}
        if public:
            data = client.get("/playbooks/public", params=params).json()
            data["rows"] = [row for row in data["rows"] if row["uri"] != "satori://.satori.yml"]
        else:
            if monitor:
                params["monitor"] = monitor
            data = client.get("/playbooks", params=params).json()

        if public and not kwargs["json"]:
            sast_list = list(filter(lambda x: not bool(x.get("parameters")), data["rows"]))
            dast_list = list(filter(lambda x: bool(x.get("parameters")), data["rows"]))
            
            widths = self.calculate_widths(sast_list, dast_list)
            
            console.rule("SAST")
            autotable(
                [
                    {
                        "uri": x["uri"], 
                        "name": x["name"][:widths['sast'][1]] + "..." if len(x["name"]) > widths['sast'][1] else x["name"], 
                        "image": self.truncate_image_name(x["image"], widths['sast'][2])
                    }
                    for x in sast_list
                ],
                widths=widths['sast']
            )
            
            console.rule("DAST")
            autotable(
                [
                    {
                        "uri": x["uri"],
                        "name": x["name"][:widths['dast'][1]] + "..." if len(x["name"] or "") > widths['dast'][1] else x["name"],
                        "parameters": ("\n".join(x["parameters"]))[:widths['dast'][2]] + "..." if len("\n".join(x["parameters"])) > widths['dast'][2] else "\n".join(x["parameters"]),
                        "image": self.truncate_image_name(x["image"], widths['dast'][3])
                    }
                    for x in dast_list
                ],
                widths=widths['dast']
            )
            
        elif kwargs["json"]:
            autoformat(data["rows"], jsonfmt=kwargs["json"], list_separator="-" * 48)
        else:
            autotable(BootstrapTable(**data), limit=limit, page=page, widths=(16,))


import json
import os
import platform
import re
import shlex
import sys
import time
from argparse import ArgumentParser
from dataclasses import asdict
from pathlib import Path
from typing import Literal, Optional, Union, get_args

import httpx
import yaml
from rich.progress import Progress, SpinnerColumn, TextColumn, TimeElapsedColumn
from satori_runner import run

from satoricli.api import client
from satoricli.bundler import make_bundle
from satoricli.cli.commands.report import ReportCommand
from satoricli.cli.utils import log
from satoricli.validations import validate_parameters

from ..utils import (
    console,
    detect_boolean,
    error_console,
    load_cli_params,
    missing_ymls,
    output_to_string,
    print_output,
    print_summary,
    tuple_to_dict,
    validate_config,
)
from .base import BaseCommand

IS_LINUX = platform.system() == "Linux"
VISIBILITY_VALUES = Literal["public", "private", "unlisted"]
FUNCTIONS_RE = re.compile(r"(read|trim|strip)\((.+)\)")
FUNCTIONS_SUB_RE = re.compile(r"(.+)\${{(.+)}}(.+)?")


def rebuild_arguments() -> str:
    """Rebuild original console arguments.

    Returns:
        str: The rebuilt arguments.

    """
    args = sys.argv[1:]
    new_args = []
    input_data_regex = re.compile(r"\S+\=.+")
    data_regex = re.compile(r"(--data(-file)?\=?)([^\-]+)")
    for arg in args:
        if input_data_regex.match(arg):
            new_args.append(f'"{arg}"')
        elif data_regex.match(arg):
            new_args.append(data_regex.sub("\\1'\\3'", arg))
        else:
            new_args.append(arg)
    return " ".join(new_args)


def new_local_run(
    team: str,
    bundle=None,
    playbook_uri: Optional[str] = None,
    secrets: Optional[dict] = None,
    name: Optional[str] = None,
    visibility: Optional[VISIBILITY_VALUES] = None,
    redacted: list[str] = [],
    run_tests: list[str] = [],
) -> dict:
    """Create a new local run.

    Returns:
        dict: The response from the API.
        {"report_id":"<id>","recipe":"<url>","token":"Bearer <token>"}

    """
    return client.post(
        "/runs/local",
        data={
            "secrets": json.dumps(secrets) if secrets else None,
            "playbook_uri": playbook_uri,
            "name": name,
            "team": team,
            "run_params": rebuild_arguments(),
            "visibility": visibility.capitalize() if visibility else None,
            "redacted": redacted,
            "run_tests": run_tests,
        },
        files={"bundle": bundle} if bundle else {"": ""},
    ).json()


def replace_variables(
    value: Union[str, list[str]], testcase: dict[str, str]
) -> Union[list, bytes, str]:
    if isinstance(value, str):
        for param_key, param_value in testcase.items():
            if m := FUNCTIONS_RE.search(param_value):
                value = execute_functions(value, m.group(1), m.group(2))
            else:
                value = replace_params(value, param_key, param_value)
        return value.encode(errors="ignore") if IS_LINUX else value
    else:
        for param_key, param_value in testcase.items():
            if m := FUNCTIONS_RE.search(param_value):
                # Run as shell if satori function is used
                value = execute_functions(" ".join(value), m.group(1), m.group(2))
                return value.encode(errors="ignore") if IS_LINUX else value
            for i, arg in enumerate(value):
                value[i] = replace_params(arg, param_key, param_value)
        return [arg.encode(errors="ignore") for arg in value] if IS_LINUX else value


def replace_params(old: str, secret_id: str, secret_value: str) -> str:
    return old.replace("${{" + secret_id + "}}", secret_value)


def execute_functions(original: str, function: str, param: str) -> str:
    if function == "read":
        safe_param = shlex.quote(param)
        original = re.sub(r"do\s*\n", "do ", original)
        original = re.sub(r"\n\s*", ";", original)
        original = original.replace("'", '"')
        return FUNCTIONS_SUB_RE.sub(
            f"cat {safe_param} | xargs -IX bash -c '\\1X\\3'", original
        )
    else:
        return original


def timeout_type(arg: str):
    timeout = int(arg)

    if timeout < 1:
        raise ValueError("Timeout must be greater than 0")

    return timeout


class LocalCommand(BaseCommand):
    name = "local"

    def register_args(self, parser: ArgumentParser):
        parser.add_argument("target", metavar="TARGET", default=".", nargs="?")
        parser.add_argument(
            "-p",
            "--playbook",
            help="if TARGET is a directory this playbook will be used",
        )
        parser.add_argument(
            "-d", "--data", type=load_cli_params, action="append", default=[]
        )
        parser.add_argument("--report", action="store_true")
        parser.add_argument("--output", action="store_true")
        parser.add_argument("-s", "--sync", action="store_true", help="Summary")
        parser.add_argument("--timeout", type=timeout_type)
        parser.add_argument("--name", type=str)
        parser.add_argument("--save-report", type=str, default=None)
        parser.add_argument("--save-output", type=str, default=None)
        parser.add_argument(
            "--visibility", choices=get_args(VISIBILITY_VALUES), default=None
        )
        parser.add_argument(
            "-df", "--data-file", type=load_cli_params, action="append", default=[]
        )
        parser.add_argument(
            "--test",
            dest="filter_tests",
            help="Print specified test output",
            action="append",
            default=[],
        )
        parser.add_argument(
            "--run",
            dest="run_tests",
            help="Run specific tests",
            action="append",
            default=[],
        )
        parser.add_argument(
            "--format",
            dest="text_format",
            help="Format text output (Plain or Markdown text)",
            default="plain",
            choices=("plain", "md"),
        )
        parser.add_argument(
            "--redacted",
            dest="redacted",
            help="Redacted parameters",
            action="append",
            default=[],
        )

    def __call__(
        self,
        target: str,
        data: list[tuple[str, str]],
        data_file: list[tuple[str, str]],
        playbook: Optional[str],
        report: bool,
        output: bool,
        sync: bool,
        timeout: Optional[int],
        name: str,
        team: str,
        save_report: Union[str, bool, None],
        save_output: Union[str, bool, None],
        visibility: Optional[VISIBILITY_VALUES],
        filter_tests: list,
        run_tests: list,
        text_format: Literal["plain", "md"],
        redacted: list[str],
        **kwargs,
    ):
        for file in data_file:
            data.append((file[0], f"read({file[1]})"))
        parsed_data = tuple_to_dict(data) if data else None
        if parsed_data and not validate_parameters(parsed_data):
            raise ValueError("Malformed parameters")

        workdir = Path(target) if os.path.isdir(target) else Path()
        config = None

        if (playbook and "://" in playbook) or "://" in target:
            bundle = None
        elif (playbook and os.path.isfile(playbook)) or os.path.isfile(target):
            path = Path(playbook or target)
            config = yaml.safe_load(path.read_bytes())

            if not validate_config(
                path, set(parsed_data.keys()) if parsed_data else set()
            ):
                return 1

            bundle = make_bundle(path, path.parent)
        elif os.path.isdir(target):
            playbook_path = workdir / ".satori.yml"
            config = yaml.safe_load(playbook_path.read_bytes())

            if not validate_config(
                playbook_path, set(parsed_data.keys()) if parsed_data else set()
            ):
                return 1

            bundle = make_bundle(playbook_path, playbook_path.parent)

            if missing_ymls(config, str(workdir)):
                log.warning(
                    "There are some .satori.yml outside the root "
                    "folder that have not been imported."
                )
        else:
            error_console.print("ERROR: Invalid args")
            return 1

        local_run = new_local_run(
            team,
            bundle,
            secrets=parsed_data,
            name=name,
            visibility=visibility,
            playbook_uri=playbook or target,
            redacted=redacted,
            run_tests=run_tests,
        )

        save_output_setting = True
        save_report_setting = True
        if config and config.get("settings"):
            if config["settings"].get("saveOutput") is not None:
                save_output_setting = config["settings"]["saveOutput"]
            if config["settings"].get("saveReport") is not None:
                save_report_setting = config["settings"]["saveReport"]
        save_output = (
            detect_boolean(save_output) if save_output else save_output_setting
        )
        save_report = (
            detect_boolean(save_report) if save_report else save_report_setting
        )

        report_id = local_run["report_id"]

        if not kwargs["json"]:
            error_console.print("Report ID:", report_id)
            error_console.print(f"Report: https://satori.ci/report/{report_id}")
        elif kwargs["json"]:
            console.print_json(data={"report_id": report_id})

        with (
            httpx.stream("GET", local_run["recipe"]) as s,
            Progress(
                SpinnerColumn("dots2"),
                TextColumn("[progress.description]Status: {task.description}"),
                TimeElapsedColumn(),
                console=error_console,
            ) as progress,
        ):
            os.chdir(workdir)
            task = progress.add_task("Starting execution")
            start_time = time.monotonic()
            deadline = start_time + timeout if timeout is not None else None

            headers = {"Authorization": "Bearer " + local_run["token"]}

            timed_out = False

            for line in s.iter_lines():
                if deadline and time.monotonic() > deadline:
                    timed_out = True
                    break

                message: dict = json.loads(line)

                progress.update(task, description="Running [b]" + message["path"])
                args = replace_variables(message["value"], message["testcase"])
                command_timeout = message.get("settings", {}).get("setCommandTimeout")

                if deadline:
                    command_timeout = (
                        min(deadline - time.monotonic(), command_timeout)
                        if command_timeout is not None
                        else deadline - time.monotonic()
                    )

                out = run(args, command_timeout)
                output_dict = asdict(out)
                output_dict["stdout"] = output_to_string(out.stdout)
                output_dict["stderr"] = output_to_string(out.stderr)
                output_dict["os_error"] = output_to_string(out.os_error)
                result = {
                    "path": message.pop("path"),
                    **output_dict,
                    "command": message,
                }
                client.post("outputs/upload", json=result, headers=headers)

            client.put(
                "/runs/local/upload",
                params={
                    "run_time": time.monotonic() - start_time,
                    "save_report": save_report,
                    "save_output": save_output,
                    "timed_out": timed_out,
                },
                headers=headers,
            )

            client.patch("/reports/severities", headers=headers)
            progress.update(task, description="Timeout" if timed_out else "Completed")

        if sync:
            print_summary(report_id, kwargs["json"])

        if report:
            ReportCommand.print_report_asrt(report_id, kwargs["json"])

        if output:
            print_output(report_id, kwargs["json"], filter_tests, text_format)

        report_data = client.get(f"/reports/{report_id}").json()

        if run_tests:
            # clean report data if --run is used
            client.delete(f"/reports/{report_id}")

        return 0 if report_data["fails"] == 0 else 1



from argparse import ArgumentParser
from typing import Literal, Optional, get_args

from satoricli.api import client
from satoricli.cli.utils import (
    VISIBILITY_VALUES,
    BootstrapTable,
    autoformat,
    autotable,
    console,
    error_console,
    get_offset,
)

from .base import BaseCommand
from .dashboard import DashboardCommand
from .report import ReportCommand
from .settings import SettingsCommand

ACTIONS = Literal[
    "show",
    "create",
    "members",
    "repos",
    "monitors",
    "reports",
    "get_config",
    "set_config",
    "del",
    "delete",
    "add",
    "settings",
    "playbooks",
    "visibility",
    "logs",
]


class TeamCommand(BaseCommand):
    name = "team"

    def register_args(self, parser: ArgumentParser):
        parser.add_argument("--role", default="READ", help="User role")

        # Use with the add command
        parser.add_argument("--repo", help="Repo name")
        parser.add_argument("--email", help="User email")
        parser.add_argument("--github", help="Github user name")
        parser.add_argument("--monitor", help="Monitor id")

        parser.add_argument("id")
        parser.add_argument(
            "action", nargs="?", choices=get_args(ACTIONS), default="show"
        )
        parser.add_argument("config_name", nargs="?")
        parser.add_argument("config_value", nargs="?")
        parser.add_argument("-p", "--page", type=int, default=1)
        parser.add_argument("-l", "--limit", type=int, default=20)

    def __call__(
        self,
        id: str,
        action: ACTIONS,
        role: Optional[str],
        repo: Optional[str],
        email: Optional[str],
        github: Optional[str],
        monitor: Optional[str],
        config_name: Optional[str],
        config_value: Optional[str],
        page: int,
        limit: int,
        **kwargs,
    ) -> int | None:
        if action == "show":
            info = client.get(f"/teams/{id}").json()
            return DashboardCommand.generate_dashboard(info)
        elif action == "create":
            info = client.post(f"/teams/{id}").json()
        elif action == "members":
            info = client.get(f"/teams/{id}/members").json()
        elif action == "add":
            if email:
                info = client.post(
                    f"/teams/{id}/members", json={"email": email, "role": role}
                ).json()
            elif github:
                info = client.post(
                    f"/teams/{id}/members", json={"github": github, "role": role}
                ).json()
            elif repo:
                info = client.post(f"/teams/{id}/repos", json={"repo": repo}).json()
            elif monitor:
                info = client.post(
                    f"/teams/{id}/monitors", json={"monitor": monitor}
                ).json()
            else:
                raise Exception("Use --github, --email, --repo or --monitor")
        elif action == "repos":
            offset = get_offset(page, limit)
            info = client.get(
                f"/teams/{id}/repos", params={"offset": offset, "limit": limit}
            ).json()
            autotable(
                BootstrapTable(**info), "b blue", False, (20, 20, None), page, limit
            )
            return 0
        elif action == "get_config":
            info = client.get(f"/teams/{id}/config/{config_name}").text
        elif action == "set_config":
            info = client.put(
                f"/teams/{id}/config",
                json={"name": config_name, "value": config_value},
            ).json()
        elif action == "del":
            if email:
                client.request(
                    "DELETE", f"/teams/{id}/members", params={"email": email}
                )
                console.print("Team member deleted")
            elif repo:
                client.request("DELETE", f"/teams/{id}/repos", json={"repo": repo})
                console.print("Team repo deleted")
            elif github:
                client.request(
                    "DELETE", f"/teams/{id}/members", params={"github": github}
                )
                console.print("Team member deleted")
            elif monitor:
                client.request("DELETE", f"/teams/{id}/monitors", params={"id": monitor})
                console.print("Team monitor deleted")
            else:
                raise Exception("Use --github, --email, --repo or --monitor")
            return
        elif action == "delete":
            client.delete(f"/teams/{id}")
            console.print("Team deleted")
            return
        elif action == "monitors":
            info = client.get(f"/teams/{id}/monitors").json()
            if not kwargs["json"]:
                autotable(info["rows"], "b blue", widths=(20, 20, None))
                return
        elif action == "reports":
            info = client.get(f"/teams/{id}/reports").json()
            return ReportCommand.print_report_list(info["rows"])
        elif action == "settings":
            SettingsCommand().run_settings(team=id)
            return
        elif action == "playbooks":
            info = client.get(f"/teams/{id}/playbooks").json()
            if not kwargs["json"]:
                autotable(info, "b blue")
                return
        elif action == "visibility":
            if not config_name:
                current = client.get(f"/teams/{id}/visibility").text
                console.print(f"The current visibility is [b]{current}")
                return 0
            if config_name in VISIBILITY_VALUES:
                info = client.patch(
                    f"/teams/{id}/visibility",
                    json={"visibility": config_name.capitalize()},
                ).json()
                console.print(
                    f"The visibility has been set to [b]{config_name.capitalize()}"
                )
            else:
                error_console.print(
                    f"Allowed values for visibility: {VISIBILITY_VALUES}"
                )
                return 1
        elif action == "logs":
            info = client.get(f"/teams/{id}/logs").json()

        autoformat(info, jsonfmt=kwargs["json"], list_separator="*" * 48, table=True)
        return 0



import re
import sys
import time
from argparse import ArgumentParser
from datetime import date
from pathlib import Path
from typing import Literal, Optional, get_args

from satoricli.api import client
from satoricli.cli.commands.report import ReportCommand
from satoricli.cli.utils import (
    BootstrapTable,
    autoformat,
    autotable,
    console,
    error_console,
    execution_time,
    get_report_output_cmd_index,
    print_output,
    remove_keys_list_dict,
    wait,
)

from ..arguments import date_args
from .base import BaseCommand

SCANID_REGEX = re.compile(r"s\w{15}")
VISIBILITY_VALUES = Literal["public", "private", "unlisted"]


class ScanCommand(BaseCommand):
    name = "scan"
    options = (date_args,)

    def register_args(self, parser: ArgumentParser):
        parser.add_argument("repository", metavar="REPOSITORY")
        parser.add_argument(
            "action",
            metavar="ACTION",
            choices=(
                "new",
                "stop",
                "status",
                "reports",
                "clean",
                "check-forks",
                "check-commits",
                "visibility",
                "delete",
            ),
            nargs="?",
            default="new",
            help="action to perform",
        )
        parser.add_argument("action2", nargs="?", default=None)
        parser.add_argument(
            "-c", "--coverage", type=float, default=1, help="coverage"
        )
        parser.add_argument("--skip-check", action="store_true")
        parser.add_argument("--delete-commits", action="store_true")
        parser.add_argument("-s", "--sync", action="store_true")
        parser.add_argument("-o", "--output", action="store_true")
        parser.add_argument("-r", "--report", action="store_true")
        parser.add_argument("-d", "--data", help="Secrets", default="")
        parser.add_argument("-b", "--branch", default="main", help="Repo branch")
        # parser.add_argument("--filter", help="Filter names")
        parser.add_argument("--playbook", help="Playbook url or file")
        parser.add_argument("-p", "--page", type=int, default=1)
        parser.add_argument("-l", "--limit", type=int, default=20)
        parser.add_argument(
            "--visibility",
            choices=get_args(VISIBILITY_VALUES),
            default="private",
        )

    def __call__(
        self,
        repository: str,
        action: Literal[
            "new",
            "stop",
            "status",
            "reports",
            "clean",
            "check-forks",
            "check-commits",
            "visibility",
            "delete",
        ],
        action2: Optional[str],
        coverage: float,
        sync: bool,
        delete_commits: bool,
        data: Optional[str],
        branch: str,
        from_date: Optional[date],
        to_date: Optional[date],
        playbook: Optional[str],
        page: int,
        limit: int,
        team: str,
        output: bool,
        report: bool,
        visibility: VISIBILITY_VALUES = "private",
        **kwargs,
    ):
        if SCANID_REGEX.match(repository) and action == "new":
            # show scan status instead of trying to create a new scan if is a scan id
            action = "status"

        if action == "new":
            config = None

            if playbook:
                if (path := Path(playbook)) and path.is_file():
                    config = path.read_text()

                if playbook.startswith("satori://"):
                    config = playbook

            info = client.post(
                "/scan",
                json={
                    "url": repository,
                    "data": data,
                    "playbook": config,
                    "coverage": coverage,
                    "from": from_date,
                    "to": to_date,
                    "branch": branch,
                    "team": team,
                    "run_params": " ".join(sys.argv[1:]),
                    "run_last": coverage == 0.0,
                    "visibility": visibility.capitalize(),
                },
            ).json()
            if sync or output or report:
                return self.scan_sync(
                    info["id"],
                    kwargs,
                    output,
                    report,
                    total_commits=1,
                )
        elif action == "clean":
            client.delete(
                f"scan/{repository}/reports", params={"delete_commits": delete_commits}
            )
            info = {"message": "Scan reports cleaned"}
        elif action == "delete":
            client.delete(f"scan/{repository}")
            info = {"message": "Scan deleted"}
        elif action == "stop":
            self.check_scan_id(repository)
            info = client.get(f"/scan/stop/{repository}").json()
        elif action == "status":
            self.check_scan_id(repository)
            if sync:
                return self.scan_sync(repository, kwargs, output, report)
            else:
                info = client.get(f"/scan/status/{repository}").json()
        elif action == "reports":
            self.check_scan_id(repository)
            info = client.get(
                f"/scan/{repository}/reports", params={"limit": limit, "page": page}
            ).json()
            if not kwargs["json"]:
                info["rows"] = remove_keys_list_dict(info["rows"], ("fails", "created"))
                info["rows"] = [
                    {**x, "execution_time": execution_time(x["execution_time"])}
                    for x in info["rows"]
                ]
                autotable(BootstrapTable(**info), page=page, limit=limit)
                return
        elif action == "check-forks":
            info = client.get(f"/scan/{repository}/check-forks").json()
        elif action == "check-commits":
            console.print(f"Checking the list of commits of the repo {repository}")
            info = client.post(
                "/scan/check-commits", json={"repo": repository, "branch": branch}
            ).json()
            if sync:
                return self.scan_sync(repository, kwargs)
        elif action == "visibility":
            if not action2 or action2 not in VISIBILITY_VALUES:
                error_console.print(
                    f"Allowed values for visibility: {VISIBILITY_VALUES}"
                )
                return 1
            info = client.patch(
                f"/scan/{repository}", json={"visibility": action2.capitalize()}
            ).json()
        autoformat(info, jsonfmt=kwargs["json"], list_separator="-" * 48)
        return 0

    @staticmethod
    def scan_sync(
        scan_id: str,
        kwargs: dict,
        output: bool = False,
        report: bool = False,
        filter_tests: Optional[list] = None,
        text_format: Literal["plain", "md"] = "plain",
        total_commits: Optional[int] = None,
    ) -> None:
        live_output = not report
        report_index, output_index = get_report_output_cmd_index()
        if total_commits and total_commits > 1:
            live_output = False
            error_console.print(f"Scan ID: {scan_id}")
            error_console.print(f"Scan: https://satori.ci/scan/{scan_id}")
        report_id_printed = False
        while True:
            res = client.get(f"/scan/{scan_id}/reports", params={"limit": 120}).json()
            if total_commits != res["total"]:
                # If not all commits are processed, wait for 1 second and continue
                time.sleep(1)
                continue
            if res["total"] == 1 and not report_id_printed:
                # Print the report id only once
                report_id_printed = True
                report_id = res["rows"][0]["id"]
                error_console.print("Report ID:", report_id)
                error_console.print(f"Report: https://satori.ci/report/{report_id}")

            if res["rows"]:
                for row in res["rows"]:
                    console.print(f"[b gold3]Repository:[/] [b]{row['repo']}[/]")
                    wait(row["id"], live_output, filter_tests, text_format)
                    should_print_output = output and not live_output
                    print_args: tuple[str, bool] = (row["id"], kwargs["json"])
                    if report_index < output_index:
                        if report:
                            ReportCommand.print_report_asrt(*print_args)
                        if should_print_output:
                            print_output(*print_args, filter_tests, text_format)
                    else:
                        if should_print_output:
                            print_output(*print_args, filter_tests, text_format)
                        if report:
                            ReportCommand.print_report_asrt(*print_args)
                break
            time.sleep(1)

    def check_scan_id(self, scan_id: str) -> None:
        if not SCANID_REGEX.match(scan_id):
            raise Exception("Please enter a scan ID")

    def check_repo_id(self, repo_id: str) -> None:
        if not re.match(r"[^/]+/[^/]+", repo_id):
            raise Exception("Please enter a repo ID")



import fcntl
from math import ceil
import os
import select
import signal
import struct
import sys
import termios
import time
import tty
from argparse import ArgumentParser
from typing import Mapping

import httpx
import paramiko
from paramiko.ssh_exception import NoValidConnectionsError
from rich.progress import Progress

from satoricli.api import client as v1_client
from satoricli.cli.utils import autotable, console

from .base import BaseCommand


def get_terminal_size():
    buf = fcntl.ioctl(sys.stdout, termios.TIOCGWINSZ, b"\x00" * 8)
    rows, cols = struct.unpack("hh", buf[:4])
    return cols, rows


def interactive_shell(host: str, token: str):
    ssh_client = paramiko.SSHClient()
    ssh_client.set_missing_host_key_policy(paramiko.AutoAddPolicy)

    tries = 0

    while tries < 3:
        try:
            ssh_client.connect(
                hostname=host,
                username="root",
                password=token,
                look_for_keys=False,
                allow_agent=False,
            )
            break
        except NoValidConnectionsError:
            time.sleep(1)
            tries += 1

    if tries >= 3:
        print("Can't connect to host")
        sys.exit(1)

    if transport := ssh_client.get_transport():
        transport.set_keepalive(30)

    cols, rows = get_terminal_size()
    channel = ssh_client.invoke_shell(term="xterm-256color", width=cols, height=rows)

    def resize_handler(signum, frame):
        cols, rows = get_terminal_size()
        channel.resize_pty(width=cols, height=rows)

    old_handler = signal.signal(signal.SIGWINCH, resize_handler)
    old_tty = termios.tcgetattr(sys.stdin)

    try:
        tty.setraw(sys.stdin.fileno())
        channel.settimeout(0.0)

        while True:
            r, _, _ = select.select([channel, sys.stdin], [], [])

            if channel in r:
                try:
                    data = channel.recv(1024)

                    if not data:
                        break

                    sys.stdout.buffer.write(data)
                    sys.stdout.buffer.flush()
                except Exception:
                    break

            if sys.stdin in r:
                data = os.read(sys.stdin.fileno(), 1024)
                channel.send(data)
    finally:
        termios.tcsetattr(sys.stdin, termios.TCSADRAIN, old_tty)
        signal.signal(signal.SIGWINCH, old_handler)
        channel.close()
        ssh_client.close()


def remove_none_values(d: Mapping):
    return {k: v for k, v in d.items() if v is not None}


class ShellCommand(BaseCommand):
    name = "shell"

    def help(self) -> str:
        return "Launch an interactive remote shell container"

    def register_args(self, parser: ArgumentParser):
        parser.add_argument("--image", help="Docker image for the container")
        parser.add_argument(
            "--memory", type=int, help="Memory allocation (AWS Fargate)"
        )
        parser.add_argument("--cpu", type=int, help="CPU allocation (AWS Fargate)")
        parser.add_argument(
            "--region", action="append", default=[], help="AWS region (repeatable)"
        )
        parser.add_argument(
            "--timeout", type=int, help="Session duration limit in seconds"
        )

    def __call__(self, cpu, memory, image, region, timeout, **kwargs):
        client = httpx.Client(
            base_url="https://api-v2.satori.ci",
            headers=v1_client.headers,
        )

        container_settings = remove_none_values(
            {"cpu": cpu, "memory": memory, "image": image}
        )

        res = client.post(
            "/ssh_sessions",
            json={
                "regions": list(region),
                "container_settings": container_settings,
                "timeout": timeout,
            },
        )
        res.raise_for_status()
        id = res.json()["id"]

        console.print("SSH session id:", id)

        with Progress(transient=True) as progress:
            progress.add_task("Waiting for host", total=None)

            while True:
                try:
                    res = client.get(f"/ssh_sessions/{id}")
                    res.raise_for_status()
                    session_data = res.json()
                    break
                except Exception:
                    time.sleep(2)

        console.print(f"Connecting to {session_data['host']}")

        interactive_shell(session_data["host"], session_data["token"])


class ShellsCommand(BaseCommand):
    name = "shells"

    def register_args(self, parser: ArgumentParser):
        parser.add_argument("-q", "--quantity", type=int, default=10)
        parser.add_argument("-p", "--page", type=int, default=1)

    def __call__(self, page: int, quantity: int, **kwargs):
        client = httpx.Client(
            base_url="https://api-v2.satori.ci",
            headers=v1_client.headers,
        )

        res = client.get("/ssh_sessions", params={"page": page, "quantity": quantity})
        res.raise_for_status()

        data = res.json()

        def clean_result(session: dict):
            session.pop("container_settings")
            session["regions"] = ", ".join(session["regions"])
            session["created_at"] = session["created_at"].split(".")[0]

            if session["finished_at"]:
                session["finished_at"] = session["finished_at"].split(".")[0]

            return session

        autotable([clean_result(s) for s in data["items"]])

        console.print(
            f"Page {page} of {ceil(data['total'] / quantity)} | Total: {data['total']}"
        )



from argparse import ArgumentParser

from satoricli.cli.utils import console
from satoricli.utils import load_config

from .base import BaseCommand


class WidthCommand(BaseCommand):
    name = "width"

    def register_args(self, parser: ArgumentParser):
        pass  # No arguments needed, just show current width

    def __call__(self, **kwargs):
        config = load_config()
        profile = kwargs.get("profile", "default")

        profile_config = config.get(profile, {})
        current_width = profile_config.get("width")

        if current_width is not None:
            console.print(f"Current configured width for profile '{profile}': {current_width}")
        else:
            console.print(f"Width not configured for profile '{profile}'. Console width is auto-detected.")